/***************************************************************************
*   Copyright 2007 Niko Sams <niko.sams@gmail.com>                        *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/
#include "uploaddialog.h"
#include "ui_uploaddialog.h"
#include "uploadprojectmodel.h"
#include "kdevuploadplugin.h"
#include "uploadprofilemodel.h"
#include "uploadprofileitem.h"
#include "uploadprofiledlg.h"

#include <QPushButton>
#include <QHeaderView>

#include <kconfiggroup.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kio/job.h>
#include <kio/copyjob.h>
#include <kio/jobuidelegate.h>

#include <icore.h>
#include <iproject.h>
#include <iprojectcontroller.h>
#include <projectmodel.h>


UploadDialog::UploadDialog( UploadPlugin* plugin, KDevelop::IProject* project, QWidget *parent )
    : QDialog(parent), m_plugin(plugin), m_project(project), m_editProfileDlg(0)
{
    m_ui = new Ui::UploadDialog();
    m_ui->setupUi(this);
    m_ui->buttonBox->button(QDialogButtonBox::Apply)->setText(i18n("&Upload"));
    connect(m_ui->buttonBox, SIGNAL(clicked(QAbstractButton*)),
            this, SLOT(buttonClicked(QAbstractButton*)));

    m_uploadProjectModel = new UploadProjectModel(project);
    m_uploadProjectModel->setSourceModel(m_plugin->core()->projectController()->projectModel());
    m_ui->projectTree->setModel(m_uploadProjectModel);
    m_ui->projectTree->header()->hide();

    m_profileModel = new UploadProfileModel;
    m_profileModel->setConfig(m_project->projectConfiguration()->group("Upload"));
    m_ui->profileCombobox->setModel(m_profileModel);

    connect(m_ui->profileCombobox, SIGNAL(currentIndexChanged(int)),
            this, SLOT(profileChanged(int)));

    m_ui->profileCombobox->setCurrentIndex(-1);
    for (int i = 0; i < m_profileModel->rowCount(); i++) {
        if (m_profileModel->uploadItem(i)->isDefault()) {
            m_ui->profileCombobox->setCurrentIndex(i);
            break;
        }
    }

    m_ui->progress->hide();
    m_ui->infoMessage->hide();

    connect(m_ui->modifyProfileButton, SIGNAL(clicked()),
            this, SLOT(modifyProfile()));
}

UploadDialog::~UploadDialog()
{
    delete m_ui;
    delete m_editProfileDlg;
}

void UploadDialog::modifyProfile()
{
    UploadProfileItem* i = m_profileModel->uploadItem(m_ui->profileCombobox->currentIndex());
    if (i) {
        if (!m_editProfileDlg) {
            m_editProfileDlg = new UploadProfileDlg(this);
        }
        if (m_editProfileDlg->editProfile(i) == QDialog::Accepted) {
            m_profileModel->submit();
        }
    }
}

void UploadDialog::profileChanged(int index)
{
    UploadProfileItem* i = m_profileModel->uploadItem(index);
    if (i) {
        KConfigGroup c = i->profileConfigGroup();
        if (c.isValid()) {
            m_uploadProjectModel->setProfileConfigGroup(c);
            m_ui->projectTree->setEnabled(true);
            m_ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(true);
            return;
        }
    }
    m_ui->projectTree->setEnabled(false);
    m_ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
}

void UploadDialog::buttonClicked(QAbstractButton* button)
{
    
    if (button == m_ui->buttonBox->button(QDialogButtonBox::Apply)) {
        if (m_ui->profileCombobox->currentIndex() == -1) {
            KMessageBox::sorry(this, i18n("Can't upload, no profile selected."));
            return;
        }
        m_ui->progress->setValue(0);
        m_ui->progress->show();
        m_ui->infoMessage->setText("");
        m_ui->infoMessage->show();
        m_uploadIndex = QModelIndex();
        uploadNext();
    }
}
void UploadDialog::uploadNext()
{
    m_uploadIndex = m_uploadProjectModel->nextRecursionIndex(m_uploadIndex);

    if (!m_uploadIndex.isValid()) {
        //last index reached - finished
        accept();
        return;
    }

    KDevelop::ProjectBaseItem* item = m_uploadProjectModel->item(m_uploadIndex);

    Qt::CheckState checked = static_cast<Qt::CheckState>(m_uploadProjectModel
                            ->data(m_uploadIndex, Qt::CheckStateRole).toInt());
    if ((item->file() || item->folder()) && checked != Qt::Unchecked) {
        KUrl dest = m_uploadProjectModel->profileConfigGroup().readEntry("url", KUrl());

        KIO::Job* job;
        if (item->file()) {
            dest.addPath(m_project->relativeUrl(item->file()->url()).path());
            kDebug() << "file_copy" << item->file()->url() << dest;
            job = KIO::file_copy(item->file()->url(), dest, -1,
                                       KIO::Overwrite | KIO::HideProgressInfo);
        } else if (item->folder()) {
            dest.addPath(m_project->relativeUrl(item->folder()->url()).path());

            kDebug() << "stat" << dest;
            KIO::StatJob* statJob = KIO::stat(dest, KIO::HideProgressInfo);
            statJob->ui()->setWindow(this);
            statJob->setDetails(0);
            statJob->exec();
            if (!statJob->error()) {
                kDebug() << "dir exists" << dest;
                m_uploadProjectModel->profileConfigGroup()
                        .writeEntry(item->folder()->url().url(), QDateTime::currentDateTime());
                uploadNext();
                return;
            }
            if (statJob->error() == KIO::ERR_DOES_NOT_EXIST) {
                kDebug() << "mkdir" << dest;
                job = KIO::mkdir(dest);
            } else {
                handleError(statJob);
                return;
            }
        } else {
            uploadNext();
            return;
        }

        job->ui()->setWindow(this);
        connect(job, SIGNAL(result(KJob*)),
                this, SLOT(uploadResult(KJob*)));
        connect(job, SIGNAL(percent(KJob*, unsigned long)),
                this, SLOT(uploadPercent(KJob*, unsigned long)));
        connect(job, SIGNAL(infoMessage(KJob*, QString)),
                this, SLOT(uploadInfoMessage(KJob*, QString)));
        connect(this, SIGNAL(finished(int)),  //is this actually working?
                job, SLOT(kill()));
        job->start();
    } else {
        uploadNext();
    }
}

void UploadDialog::uploadResult(KJob* job)
{
    if (job->error()) {
        handleError(job);
        return;
    }
    KDevelop::ProjectBaseItem* item = m_uploadProjectModel->item(m_uploadIndex);
    QString url;
    if (item->file()) {
        url = item->file()->url().url();
    } else if (item->folder()) {
        url = item->folder()->url().url();
    }
    if (!url.isEmpty()) {
        m_uploadProjectModel->profileConfigGroup()
                .writeEntry(url, QDateTime::currentDateTime());
    }

    uploadNext();
}
void UploadDialog::handleError(KJob* job)
{
    if (job->error() == KIO::ERR_USER_CANCELED) {
        reject();
    }
    if (job->error()) {
        qobject_cast<KIO::Job*>(job)->ui()->showErrorMessage();
        return;
    }
}

void UploadDialog::uploadPercent(KJob*, unsigned long percent)
{
    m_ui->progress->setValue(percent);
}
void UploadDialog::uploadInfoMessage(KJob*, const QString& plain)
{
    m_ui->infoMessage->setText(plain);
}

void UploadDialog::setRootItem(KDevelop::ProjectBaseItem* item)
{
    m_uploadProjectModel->setRootItem(item);
    if (item) {
        QModelIndex i = m_uploadProjectModel->mapFromSource(item->index());
        while (i.isValid()) {
            m_ui->projectTree->expand(i);
            i = i.parent();
        }
    }
}

#include "uploaddialog.moc"
// kate: space-indent on; indent-width 4; tab-width 4; replace-tabs on
